==> tkabber_0.9.7.orig.tar.gz: tkabber-0.9.7/presence.tcl <==
# $Id: presence.tcl,v 1.58 2004/06/17 21:30:29 aleksey Exp $


proc client:presence {connid from type x args} {
    global presence
    global processed_presence

    debugmsg presence "PRESENCE: $from; $type; $x; $args"

    set from [tolower_node_and_domain $from]

    switch -- $type {
	error -
	unavailable {
            if {[cequal $type unavailable]} {
	        presence_process_x $connid $from $type $x
	    }

	    catch { unset presence(type,$connid,$from) }
	    catch { unset presence(status,$connid,$from) }
	    catch { unset presence(priority,$connid,$from) }
	    catch { unset presence(meta,$connid,$from) }
	    catch { unset presence(icon,$connid,$from) }
	    catch { unset presence(show,$connid,$from) }
	    catch { unset presence(loc,$connid,$from) }
	    catch { unset presence(x,$connid,$from) }
	    catch { unset presence(signed,$connid,$from) }
	    catch { unset presence(error,$connid,$from) }

	    set user [node_and_server_from_jid $from]
	    if {[info exists presence(user_jids,$connid,$user)]} {
		set idx [lsearch -exact $presence(user_jids,$connid,$user) $from]
		set presence(user_jids,$connid,$user) \
		    [lreplace $presence(user_jids,$connid,$user) $idx $idx]
	    }
	    cache_preferred_jid_on_unavailable $connid $from $user
	    cache_user_status $connid $user

	    foreach {attr val} $args {
		switch -- $attr {
		    -status {
			set presence(status,$connid,$from) $val
			if {[get_user_status $connid $user] == "unavailable"} {
			    set presence(status,$connid,$user) $val
			}
		    }
		    -error {
			set presence(error,$connid,$from) $val
		    }
		}
	    }

	    roster::on_change_jid_presence $connid $from
	    debugmsg presence "$connid $from unavailable"
	    chat::change_presence $connid $from $type
	}
	subscribe {eval [list message::show_subscribe_dialog $from $x -connection $connid] $args}
	subscribed {eval [list roster::process_subscribed $connid $from $x] $args}
	unsubscribe {}
	unsubscribed {
	    MessageDlg .unsubscribed -aspect 50000 -icon info \
		-title [format [::msgcat::mc "Unsubscribed from %s"] $from] \
		-message [format \
			      [::msgcat::mc "We unsubscribed from %s"] $from] \
		-type user -buttons {ok} -default 0 -cancel 0
	}
	probe {}
	default {
	    set presence(type,$connid,$from)     available
	    set presence(status,$connid,$from)   ""
	    set presence(priority,$connid,$from) 0
	    set presence(meta,$connid,$from)     ""
	    set presence(icon,$connid,$from)     ""
	    set presence(show,$connid,$from)     available
	    set presence(loc,$connid,$from)      ""
	    set presence(x,$connid,$from)        $x
	    
	    foreach {attr val} $args {
		switch -- $attr {
		    -status   {set presence(status,$connid,$from)   $val}
		    -priority {set presence(priority,$connid,$from) $val}
		    -meta     {set presence(meta,$connid,$from)     $val}
		    -icon     {set presence(icon,$connid,$from)     $val}
		    -show     {set presence(show,$connid,$from)     $val}
		    -loc      {set presence(loc,$connid,$from)      $val}
		}
	    }
	    
	    set presence(show,$connid,$from) [normalize_show $presence(show,$connid,$from)]
	    set show $presence(show,$connid,$from)

	    set user [node_and_server_from_jid $from]
	    if {![info exists presence(user_jids,$connid,$user)] || \
		    ![lcontain $presence(user_jids,$connid,$user) $from]} {
		lappend presence(user_jids,$connid,$user) $from
	    }

	    cache_preferred_jid_on_available $connid $from $user
	    cache_user_status $connid $user

	    presence_process_x $connid $from available $x

	    roster::on_change_jid_presence $connid $from
	    chat::change_presence $connid $from $show
	}
    }
    
    hook::run client_presence_hook $connid $from $type $x $args
}



proc get_jids_of_user {connid user} {
    global presence

    if {[info exists presence(user_jids,$connid,$user)]} {
	return $presence(user_jids,$connid,$user)
    } elseif {![cequal [resource_from_jid $user] ""]} {
	if {[info exists presence(type,$connid,$user)]} {
	    return [list $user]
	}
    }
    return {}
}

proc get_jid_of_user {connid user} {
    global presence

    if {[info exists presence(preferred_jid,$connid,$user)]} {
	return $presence(preferred_jid,$connid,$user)
    } else {
	return $user
    }
}

proc cache_preferred_jid_on_available {connid jid user} {
    global presence

    if {[info exists presence(maxpriority,$connid,$user)]} {
	set maxpri $presence(maxpriority,$connid,$user)
    } else {
	cache_preferred_jid $connid $user
	return
    }
    
    set pri $presence(priority,$connid,$jid)

    if {$pri > $maxpri} {
	set presence(maxpriority,$connid,$user) $pri
	set presence(preferred_jid,$connid,$user) $jid
    }
}

proc cache_preferred_jid_on_unavailable {connid jid user} {
    global presence

    if {![info exists presence(maxpriority,$connid,$user)]} {
	cache_preferred_jid $connid $user
	return
    }
    
    if {$presence(preferred_jid,$connid,$user) == $jid} {
	unset presence(preferred_jid,$connid,$user)
	unset presence(maxpriority,$connid,$user)
	cache_preferred_jid $connid $user
    }
}

proc cache_preferred_jid {connid user} {
    global presence

    set jids [get_jids_of_user $connid $user]

    if {$jids != {}} {
	set rjid [lindex $jids 0]
	set pri $presence(priority,$connid,$rjid)

	foreach jid $jids {
	    if {$presence(priority,$connid,$jid) > $pri} {
		set pri $presence(priority,$connid,$jid)
		set rjid $jid
	    }
	}

	set presence(maxpriority,$connid,$user) $pri
	set presence(preferred_jid,$connid,$user) $rjid
    }
}


proc get_jid_status {connid jid} {
    global presence

    set j $jid
    if {[info exists presence(show,$connid,$j)]} {
	return $presence(show,$connid,$j)
    } else {
	return unavailable
    }
}

proc get_jid_presence_info {param connid jid} {
    global presence

    if {[info exists presence($param,$connid,$jid)]} {
	return $presence($param,$connid,$jid)
    } else {
	return ""
    }
}

proc get_user_status {connid user} {
    global presence

    if {[info exists presence(cachedstatus,$connid,$user)]} {
	return $presence(cachedstatus,$connid,$user)
    } elseif {[info exists presence(show,$connid,$user)]} {
	return $presence(show,$connid,$user)
    } else {
	return unavailable
    }
}

proc cache_user_status {connid user} {
    global presence

    set jid [get_jid_of_user $connid $user]
    if {[info exists presence(show,$connid,$jid)]} {
	set presence(cachedstatus,$connid,$user) $presence(show,$connid,$jid)
    } else {
	set presence(cachedstatus,$connid,$user) unavailable
    }
}

proc get_user_status_desc {connid user} {
    global presence

    set jid [get_jid_of_user $connid $user]
    if {[info exists presence(status,$connid,$jid)]} {
	return $presence(status,$connid,$jid)
    } else {
	return ""
    }
}

array set status_priority {
    stalker     0
    unavailable 1
    xa          2
    away        3
    dnd         4
    available   5
    chat        6
}

proc compare_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 > $p2} {
	return 1
    } elseif {$p1 == $p2} {
	return 0
    } else {
	return -1
    }
}

proc max_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 >= $p2} {
	return $s1
    } else {
	return $s2
    }
}

set userpriority 0
set curuserstatus [set userstatus available]
set curtextstatus [set textstatus ""]

set userstatusdesc [::msgcat::mc "Not logged in"]

set statusdesc(available) [::msgcat::mc "Online"]
set statusdesc(chat) [::msgcat::mc "Free to chat"]
set statusdesc(away) [::msgcat::mc "Away"]
set statusdesc(xa) [::msgcat::mc "Extended away"]
set statusdesc(dnd) [::msgcat::mc "Do not disturb"]
set statusdesc(invisible) [::msgcat::mc "Invisible"]
set statusdesc(unavailable) [::msgcat::mc "Offline"]


proc change_priority_dialog {} {
    global tmppriority
    global userpriority

    set tmppriority $userpriority

    set w .change_priority
    if {[winfo exists $w]} {
	focus -force $w
	return
    }

    Dialog $w -title [::msgcat::mc "Change Presence Priority"] \
	-modal none -separator 1 -anchor e -default 0 -cancel 1 \
        -parent .

    $w add -text [::msgcat::mc "OK"] \
	-command [list do_change_priority $w]
    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
    
    set f [$w getframe]
    label $f.lpriority -text [::msgcat::mc "Priority:"]
    Spinbox $f.priority -1000 1000 1 tmppriority

    grid $f.lpriority -row 0 -column 0 -sticky e
    grid $f.priority  -row 0 -column 1 -sticky ew

    grid columnconfigure $f 0 -weight 1 
    grid columnconfigure $f 1 -weight 1

    $w draw

}

proc do_change_priority {w} {
    global tmppriority
    global userpriority

    destroy $w
    if {![cequal $userpriority $tmppriority]} {
        set userpriority $tmppriority
        change_our_presence userpriority {} w
    }
}

trace variable userstatus w change_our_presence

proc change_our_presence {name1 name2 op} {
    global userstatus curuserstatus statusdesc userstatusdesc
    global textstatus curtextstatus
    global userpriority

    if {(![cequal $userstatus $curuserstatus]) \
	    || (![cequal $textstatus $curtextstatus]) \
	    || [cequal $name1 userpriority]} {
	if {[lsearch -exact [array names statusdesc] $userstatus] < 0} {
	    error [cconcat [::msgcat::mc "invalid userstatus value "] $userstatus]
	}

	set userstatusdesc $statusdesc($userstatus)
	set status $textstatus
	if {$::ssj::options(sign-traffic) && [cequal $status {}]} {
	    set status $userstatusdesc
	}

	set p $userpriority
	set x [get_our_presence_xlist $status]

	switch -- $userstatus {
	    available {set command [list jlib::send_presence \
					-xlist $x -pri $p]}
            unavailable -
	    invisible {set command [list jlib::send_presence \
					-type $userstatus -xlist $x -pri $p]}
	    default {
		set command [list jlib::send_presence \
				 -show $userstatus -xlist $x -pri $p]
	    }
	}
        if {![cequal $status ""]} {
	    lappend command -stat $status
	}
	debugmsg presence $command

	foreach connid [jlib::connections] {
	    eval $command -connection $connid
	}
	if {$userstatus != "invisible"} {
	    foreach chatid [array names chat::opened] {
		set connid [chat::get_connid $chatid]
		set group [chat::get_jid $chatid]
	        if {[cequal $chat::chats(type,$chatid) groupchat]} {
		    set nick [get_our_groupchat_nick $chatid]
		    eval $command \
			[list -to $group/$nick \
			      -connection $connid]
		}
	    }
	} else {
	    foreach chatid [array names chat::opened] {
		set connid [chat::get_connid $chatid]
		set group [chat::get_jid $chatid]
	        if {[cequal $chat::chats(type,$chatid) groupchat]} {
		    set nick [get_our_groupchat_nick $chatid]
		    jlib::send_presence \
		        -to $group/$nick \
			-connection $connid \
			-xlist $x
		}
	    }
	}

	set curuserstatus $userstatus
	set curtextstatus $textstatus
	hook::run change_our_presence_post_hook $userstatus
    }
}

proc send_first_presence {connid} {
    global userstatus curuserstatus statusdesc userstatusdesc
    global textstatus curtextstatus
    global userpriority
    global loginconf

    if {[lsearch -exact [array names statusdesc] $userstatus] < 0} {
	error [cconcat [::msgcat::mc "invalid userstatus value "] $userstatus]
    }

    set userstatusdesc $statusdesc($userstatus)
    set status $textstatus
    if {$::ssj::options(sign-traffic) && [cequal $status {}]} {
	set status $userstatusdesc
    }

    set p [set userpriority $loginconf(priority)]
    set x [get_our_presence_xlist $status]

    switch -- $userstatus {
	available {set command [list jlib::send_presence \
				    -xlist $x -pri $p]}
	unavailable -
	invisible {set command [list jlib::send_presence \
				    -type $userstatus -xlist $x -pri $p]}
	default {
	    set command [list jlib::send_presence \
			     -show $userstatus -xlist $x -pri $p]
	}
    }
    if {![cequal $status ""]} {
	lappend command -stat $status
    }
    debugmsg presence $command

    eval $command -connection $connid

    set curuserstatus $userstatus
    set curtextstatus $textstatus
    if {[cequal $userstatus unavailable]} {
	set curuserstatus available
        set userstatus available
    } else {
	set curuserstatus $userstatus
	set userstatus $userstatus
    }

    hook::run change_our_presence_post_hook $userstatus
}

hook::add connected_hook [namespace current]::send_first_presence 10

proc send_custom_presence {jid status args} {
    global statusdesc
    global userpriority

    set type jid
    foreach {opt val} $args {
	switch -- $opt {
	    -type       { set type $val }
	    -connection { set connid $val }
	}
    }
    if {![info exists connid]} {
	set connid [jlib::route $jid]
    }

    switch -- $type {
	group   { set to $jid/[get_our_groupchat_nick [chat::chatid $connid $jid]] }
	default { set to $jid }
    }

    if {$::ssj::options(sign-traffic)} {
	set stat $statusdesc($status)
    } else {
	set stat ""
    }
    set p $userpriority
    set x [get_our_presence_xlist $stat]

    switch -- $status {
	available {set command [list jlib::send_presence -to $to \
				    -xlist $x -pri $p]}
	unavailable {set command [list jlib::send_presence -to $to \
				      -type $status -xlist $x -pri $p]}
	default {
	    set command [list jlib::send_presence -to $to \
			     -show $status -xlist $x -pri $p]
	}
    }
    if {![cequal $stat ""]} {
	lappend command -stat $stat
    }

    debugmsg presence $command
    eval $command [list -connection $connid]
}

proc get_our_presence_xlist {status} {
    set xs {}

    foreach x [list [avatar::get_presence_x] \
		    [make_signature $status]] {
	if {$x != {}} {
	    lappend xs $x
	}
    }

    return $xs
}

proc normalize_show {show} {
    set res $show
    
    switch -- $show {
	away        {}
	chat        {}
    	dnd         {}
	xa          {}
	unavailable {}
	stalker     {}
	default     {set res available}
    }
    return $res
}


proc presence_process_x {connid from type xs} {
    global presence
    # TODO: use connid
    catch { unset presence(signed,$connid,$from) }
    foreach x $xs {
	jlib::wrapper:splitxml $x tag vars isempty chdata children
	switch -- [jlib::wrapper:getattr $vars xmlns] {
	    jabber:x:avatar {
		avatar::set_hash $from $children
	    }

	    jabber:x:signed {
		check_signature $connid $from $chdata
	    }

	    "http://jabber.org/protocol/muc#user" {
		muc::process_gc_user $connid $from $type $children
	    }
	}
    }
}

proc check_signature {connid from signature} {
    global presence

# in case the sender didn't check the exit code from gpg...
    if {([cequal $signature ""]) \
	    || ([cequal [info commands ::ssj::signed:input] ""])} {
	return
    }

    set presence(signed,$connid,$from) \
	[ssj::signed:input $from $signature $presence(status,$connid,$from) \
		"Presence information"]
}

proc make_signature {status} {
    if {(![cequal [info commands ::ssj::signed:output] ""]) \
	    && (![catch { ssj::signed:output $status } chdata]) \
	    && (![cequal $chdata ""])} {
	return [jlib::wrapper:createtag x -vars {xmlns jabber:x:signed} \
			-chdata $chdata]
    }
}


proc clear_presence_info {connid} {
    global presence

    if {$connid == {}} {
	array unset presence
    } else {
	# TODO
	array unset presence type,$connid,*
	array unset presence status,$connid,*
	array unset presence priority,$connid,*
	array unset presence meta,$connid,*
	array unset presence icon,$connid,*
	array unset presence show,$connid,*
	array unset presence loc,$connid,*
	array unset presence error,$connid,*
	array unset presence x,$connid,*
	array unset presence user_jids,$connid,*
	array unset presence preferred_jid,$connid,*
	array unset presence cachedstatus,$connid,*
	array unset presence maxpriority,$connid,*
    }
}
hook::add disconnected_hook clear_presence_info

==> tkabber_0.9.8.orig.tar.gz: tkabber-0.9.8/presence.tcl <==
# $Id: presence.tcl,v 1.63 2005/04/25 23:56:56 aleksey Exp $


proc client:presence {connid from type x args} {
    global presence
    global processed_presence

    debugmsg presence "PRESENCE: $from; $type; $x; $args"

    set from [tolower_node_and_domain $from]

    switch -- $type {
	error -
	unavailable {
            if {[cequal $type unavailable]} {
	        presence_process_x $connid $from $type $x
	    }

	    catch { unset presence(type,$connid,$from) }
	    catch { unset presence(status,$connid,$from) }
	    catch { unset presence(priority,$connid,$from) }
	    catch { unset presence(meta,$connid,$from) }
	    catch { unset presence(icon,$connid,$from) }
	    catch { unset presence(show,$connid,$from) }
	    catch { unset presence(loc,$connid,$from) }
	    catch { unset presence(x,$connid,$from) }
	    catch { unset presence(signed,$connid,$from) }
	    catch { unset presence(error,$connid,$from) }

	    set user [node_and_server_from_jid $from]
	    if {[info exists presence(user_jids,$connid,$user)]} {
		set idx [lsearch -exact $presence(user_jids,$connid,$user) $from]
		set presence(user_jids,$connid,$user) \
		    [lreplace $presence(user_jids,$connid,$user) $idx $idx]
	    }
	    cache_preferred_jid_on_unavailable $connid $from $user
	    cache_user_status $connid $user

	    foreach {attr val} $args {
		switch -- $attr {
		    -status {
			set presence(status,$connid,$from) $val
			if {[get_user_status $connid $user] == "unavailable"} {
			    set presence(status,$connid,$user) $val
			}
		    }
		    -error {
			set presence(error,$connid,$from) $val
		    }
		}
	    }

	    roster::on_change_jid_presence $connid $from
	    debugmsg presence "$connid $from unavailable"
	    chat::change_presence $connid $from $type
	}
	subscribe {eval [list message::show_subscribe_dialog $from $x -connection $connid] $args}
	subscribed {eval [list roster::process_subscribed $connid $from $x] $args}
	unsubscribe {}
	unsubscribed {
	    MessageDlg .unsubscribed -aspect 50000 -icon info \
		-title [format [::msgcat::mc "Unsubscribed from %s"] $from] \
		-message [format \
			      [::msgcat::mc "We unsubscribed from %s"] $from] \
		-type user -buttons {ok} -default 0 -cancel 0
	}
	probe {}
	default {
	    set presence(type,$connid,$from)     available
	    set presence(status,$connid,$from)   ""
	    set presence(priority,$connid,$from) 0
	    set presence(meta,$connid,$from)     ""
	    set presence(icon,$connid,$from)     ""
	    set presence(show,$connid,$from)     available
	    set presence(loc,$connid,$from)      ""
	    set presence(x,$connid,$from)        $x
	    
	    foreach {attr val} $args {
		switch -- $attr {
		    -status   {set presence(status,$connid,$from)   $val}
		    -priority {set presence(priority,$connid,$from) $val}
		    -meta     {set presence(meta,$connid,$from)     $val}
		    -icon     {set presence(icon,$connid,$from)     $val}
		    -show     {set presence(show,$connid,$from)     $val}
		    -loc      {set presence(loc,$connid,$from)      $val}
		}
	    }
	    
	    set presence(show,$connid,$from) [normalize_show $presence(show,$connid,$from)]
	    set show $presence(show,$connid,$from)

	    set user [node_and_server_from_jid $from]
	    if {![info exists presence(user_jids,$connid,$user)] || \
		    ![lcontain $presence(user_jids,$connid,$user) $from]} {
		lappend presence(user_jids,$connid,$user) $from
	    }

	    cache_preferred_jid_on_available $connid $from $user
	    cache_user_status $connid $user

	    presence_process_x $connid $from available $x

	    roster::on_change_jid_presence $connid $from
	    chat::change_presence $connid $from $show
	}
    }
    
    hook::run client_presence_hook $connid $from $type $x $args
}



proc get_jids_of_user {connid user} {
    global presence

    if {[info exists presence(user_jids,$connid,$user)]} {
	return $presence(user_jids,$connid,$user)
    } elseif {![cequal [resource_from_jid $user] ""]} {
	if {[info exists presence(type,$connid,$user)]} {
	    return [list $user]
	}
    }
    return {}
}

proc get_jid_of_user {connid user} {
    global presence

    if {[info exists presence(preferred_jid,$connid,$user)]} {
	return $presence(preferred_jid,$connid,$user)
    } else {
	return $user
    }
}

proc cache_preferred_jid_on_available {connid jid user} {
    global presence

    if {[info exists presence(maxpriority,$connid,$user)]} {
	set maxpri $presence(maxpriority,$connid,$user)
    } else {
	cache_preferred_jid $connid $user
	return
    }
    
    set pri $presence(priority,$connid,$jid)

    if {$pri > $maxpri} {
	set presence(maxpriority,$connid,$user) $pri
	set presence(preferred_jid,$connid,$user) $jid
    }
}

proc cache_preferred_jid_on_unavailable {connid jid user} {
    global presence

    if {![info exists presence(maxpriority,$connid,$user)]} {
	cache_preferred_jid $connid $user
	return
    }
    
    if {$presence(preferred_jid,$connid,$user) == $jid} {
	unset presence(preferred_jid,$connid,$user)
	unset presence(maxpriority,$connid,$user)
	cache_preferred_jid $connid $user
    }
}

proc cache_preferred_jid {connid user} {
    global presence

    set jids [get_jids_of_user $connid $user]

    if {$jids != {}} {
	set rjid [lindex $jids 0]
	set pri $presence(priority,$connid,$rjid)

	foreach jid $jids {
	    if {$presence(priority,$connid,$jid) > $pri} {
		set pri $presence(priority,$connid,$jid)
		set rjid $jid
	    }
	}

	set presence(maxpriority,$connid,$user) $pri
	set presence(preferred_jid,$connid,$user) $rjid
    }
}


proc get_jid_status {connid jid} {
    global presence

    set j $jid
    if {[info exists presence(show,$connid,$j)]} {
	return $presence(show,$connid,$j)
    } else {
	return unavailable
    }
}

proc get_jid_presence_info {param connid jid} {
    global presence

    if {[info exists presence($param,$connid,$jid)]} {
	return $presence($param,$connid,$jid)
    } else {
	return ""
    }
}

proc get_user_status {connid user} {
    global presence

    if {[info exists presence(cachedstatus,$connid,$user)]} {
	return $presence(cachedstatus,$connid,$user)
    } elseif {[info exists presence(show,$connid,$user)]} {
	return $presence(show,$connid,$user)
    } else {
	return unavailable
    }
}

proc cache_user_status {connid user} {
    global presence

    set jid [get_jid_of_user $connid $user]
    if {[info exists presence(show,$connid,$jid)]} {
	set presence(cachedstatus,$connid,$user) $presence(show,$connid,$jid)
    } else {
	set presence(cachedstatus,$connid,$user) unavailable
    }
}

proc get_user_status_desc {connid user} {
    global presence

    set jid [get_jid_of_user $connid $user]
    if {[info exists presence(status,$connid,$jid)]} {
	return $presence(status,$connid,$jid)
    } else {
	return ""
    }
}

array set status_priority {
    stalker     0
    unavailable 1
    xa          2
    away        3
    dnd         4
    available   5
    chat        6
}

proc compare_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 > $p2} {
	return 1
    } elseif {$p1 == $p2} {
	return 0
    } else {
	return -1
    }
}

proc max_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 >= $p2} {
	return $s1
    } else {
	return $s2
    }
}

set curpriority [set userpriority 0]
set curuserstatus [set userstatus available]
set curtextstatus [set textstatus ""]

set userstatusdesc [::msgcat::mc "Not logged in"]

set statusdesc(available) [::msgcat::mc "Available"]
set statusdesc(chat) [::msgcat::mc "Free to chat"]
set statusdesc(away) [::msgcat::mc "Away"]
set statusdesc(xa) [::msgcat::mc "Extended away"]
set statusdesc(dnd) [::msgcat::mc "Do not disturb"]
set statusdesc(invisible) [::msgcat::mc "Invisible"]
set statusdesc(unavailable) [::msgcat::mc "Unavailable"]


proc change_priority_dialog {} {
    global tmppriority
    global userpriority

    set tmppriority $userpriority

    set w .change_priority
    if {[winfo exists $w]} {
	focus -force $w
	return
    }

    Dialog $w -title [::msgcat::mc "Change Presence Priority"] \
	-modal none -separator 1 -anchor e -default 0 -cancel 1 \
        -parent .

    $w add -text [::msgcat::mc "OK"] \
	-command [list do_change_priority $w]
    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
    
    set f [$w getframe]
    label $f.lpriority -text [::msgcat::mc "Priority:"]
    Spinbox $f.priority -1000 1000 1 tmppriority

    grid $f.lpriority -row 0 -column 0 -sticky e
    grid $f.priority  -row 0 -column 1 -sticky ew

    grid columnconfigure $f 0 -weight 1 
    grid columnconfigure $f 1 -weight 1

    $w draw

}

proc do_change_priority {w} {
    global userstatus
    global tmppriority
    global userpriority

    destroy $w
    if {![cequal $userpriority $tmppriority]} {
        set userpriority $tmppriority
	set userstatus $userstatus
    }
}

trace variable userstatus w change_our_presence
trace variable logoutuserstatus w change_our_presence

proc change_our_presence {name1 name2 op} {
    global userstatus logoutuserstatus curuserstatus
    global textstatus logouttextstatus curtextstatus
    global userpriority logoutpriority curpriority
    global statusdesc userstatusdesc

    switch -- $name1 {
	logoutuserstatus {
	    set newstatus $logoutuserstatus
	    set newtextstatus $logouttextstatus
	    set newpriority $logoutpriority
	}
	default {
	    set newstatus $userstatus
	    set newtextstatus $textstatus
	    set newpriority $userpriority
	}
    }

    if {[cequal $newstatus $curuserstatus] \
	    && [cequal $newtextstatus $curtextstatus] \
	    && [cequal $newpriority $curpriority]} {
	return
    }

    if {[lsearch -exact [array names statusdesc] $newstatus] < 0} {
	error [cconcat [::msgcat::mc "invalid userstatus value "] $newstatus]
    }

    set userstatusdesc $statusdesc($newstatus)
    set status $newtextstatus
    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP] && \
	    [cequal $status {}]} {
	set status $userstatusdesc
    }

    switch -- $newstatus {
	available {
	    set command [list jlib::send_presence \
			      -pri $newpriority]
	}
        unavailable -
	invisible {
	    set command [list jlib::send_presence \
			      -type $newstatus -pri $newpriority]
	}
	default {
	    set command [list jlib::send_presence \
			      -show $newstatus -pri $newpriority]
	}
    }
    if {![cequal $status ""]} {
	lappend command -stat $status
    }
    debugmsg presence $command

    foreach connid [jlib::connections] {
	set x [hook::foldl presence_xlist_hook {} $connid $status]
	eval $command [list -xlist $x -connection $connid]
    }
    if {$newstatus != "invisible"} {
	foreach chatid [array names chat::opened] {
	    set connid [chat::get_connid $chatid]
	    set group [chat::get_jid $chatid]
	    if {[cequal $chat::chats(type,$chatid) groupchat]} {
		set nick [get_our_groupchat_nick $chatid]
		eval $command [list -to $group/$nick -connection $connid]
	    }
	}
    } else {
	foreach chatid [array names chat::opened] {
	    set connid [chat::get_connid $chatid]
	    set group [chat::get_jid $chatid]
	    if {[cequal $chat::chats(type,$chatid) groupchat]} {
		set nick [get_our_groupchat_nick $chatid]
		jlib::send_presence -to $group/$nick -xlist $x \
		    -connection $connid
	    }
	}
    }

    set curuserstatus $newstatus
    set curtextstatus $newtextstatus
    set curpriority $newpriority
    hook::run change_our_presence_post_hook $newstatus
}

proc send_first_presence {connid} {
    global userstatus curuserstatus statusdesc userstatusdesc
    global textstatus curtextstatus
    global userpriority curpriority
    global loginconf

    if {[lsearch -exact [array names statusdesc] $userstatus] < 0} {
	error [cconcat [::msgcat::mc "invalid userstatus value "] $userstatus]
    }

    set userstatusdesc $statusdesc($userstatus)
    set status $textstatus
    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP] && \
	    [cequal $status {}]} {
	set status $userstatusdesc
    }

    set curuserstatus $userstatus
    set curtextstatus $textstatus
    set curpriority [set userpriority $loginconf(priority)]
    
    set x [hook::foldl presence_xlist_hook {} $connid $status]

    switch -- $userstatus {
	available {
	    set command [list jlib::send_presence \
			      -xlist $x -pri $userpriority]
	}
	unavailable -
	invisible {
	    set command [list jlib::send_presence \
			      -type $userstatus -xlist $x -pri $userpriority]
	}
	default {
	    set command [list jlib::send_presence \
			      -show $userstatus -xlist $x -pri $userpriority]
	}
    }
    if {![cequal $status ""]} {
	lappend command -stat $status
    }
    debugmsg presence $command

    eval $command -connection $connid

    hook::run change_our_presence_post_hook $userstatus
}

hook::add connected_hook [namespace current]::send_first_presence 10

proc send_custom_presence {jid status args} {
    global statusdesc
    global userpriority

    set type jid
    foreach {opt val} $args {
	switch -- $opt {
	    -type       { set type $val }
	    -connection { set connid $val }
	}
    }
    if {![info exists connid]} {
	set connid [jlib::route $jid]
    }

    switch -- $type {
	group   { set to $jid/[get_our_groupchat_nick [chat::chatid $connid $jid]] }
	default { set to $jid }
    }

    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP]} {
	set stat $statusdesc($status)
    } else {
	set stat ""
    }

    set x [hook::foldl presence_xlist_hook {} $connid $stat]

    switch -- $status {
	available {
	    set command [list jlib::send_presence -to $to \
			      -xlist $x -pri $userpriority]
	}
	unavailable {
	    set command [list jlib::send_presence -to $to \
			      -type $status -xlist $x -pri $userpriority]
	}
	default {
	    set command [list jlib::send_presence -to $to \
			      -show $status -xlist $x -pri $userpriority]
	}
    }
    if {![cequal $stat ""]} {
	lappend command -stat $stat
    }

    debugmsg presence $command
    eval $command [list -connection $connid]
}

proc normalize_show {show} {
    set res $show
    
    switch -- $show {
	away        {}
	chat        {}
    	dnd         {}
	xa          {}
	unavailable {}
	stalker     {}
	default     {set res available}
    }
    return $res
}


proc presence_process_x {connid from type xs} {
    global presence

    catch { unset presence(signed,$connid,$from) }

    hook::run presence_process_x_hook $connid $from $type $xs
}

proc check_signature {connid from type xs} {
    global presence

    set signature ""
    foreach x $xs {
	jlib::wrapper:splitxml $x tag vars isempty chdata children
	if {[jlib::wrapper:getattr $vars xmlns] == "jabber:x:signed"} {
	    set signature $chdata
	    break
	}
    }

# in case the sender didn't check the exit code from gpg...
    if {([cequal $signature ""]) \
	    || ([cequal [info commands ::ssj::signed:input] ""])} {
	return
    }

    set presence(signed,$connid,$from) \
	[ssj::signed:input $connid $from $signature $presence(status,$connid,$from) \
		"Presence information"]
}

hook::add presence_process_x_hook [namespace current]::check_signature

proc make_signature {acc connid status} {
    if {(![cequal [info commands ::ssj::signed:output] ""]) \
	    && (![catch { ssj::signed:output $connid $status } chdata]) \
	    && (![cequal $chdata ""])} {
	lappend acc [jlib::wrapper:createtag x -vars {xmlns jabber:x:signed} \
			 -chdata $chdata]
    }
    return $acc
}

hook::add presence_xlist_hook [namespace current]::make_signature

proc clear_presence_info {connid} {
    global presence

    if {$connid == {}} {
	array unset presence
    } else {
	# TODO
	array unset presence type,$connid,*
	array unset presence status,$connid,*
	array unset presence priority,$connid,*
	array unset presence meta,$connid,*
	array unset presence icon,$connid,*
	array unset presence show,$connid,*
	array unset presence loc,$connid,*
	array unset presence error,$connid,*
	array unset presence x,$connid,*
	array unset presence user_jids,$connid,*
	array unset presence preferred_jid,$connid,*
	array unset presence cachedstatus,$connid,*
	array unset presence maxpriority,$connid,*
    }
}
hook::add disconnected_hook clear_presence_info
